//
//  IAPManager.m
//  Almond-AppVault
//
//  Created by Roman Stetsenko on 11/12/14.
//  Copyright (c) 2014 AkmStudio Ltd. All rights reserved.
//


#import "IAPManager.h"
#import "IAPHelper.h"
#import "Config.h"


NSString *const kGreatestExpiryDate = @"greatestExpiryDate";

typedef void (^IAPReceiptDataBlock) (NSData *receiptData, NSError *error);


@implementation IAPPeriod

- (NSString *)description {
    return [NSString stringWithFormat:@"%@, Start: %@, end: %@", [super description], self.startDate, self.endDate];
}

@end


@interface IAPManager () <SKRequestDelegate>

@property (nonatomic, strong) IAPHelper *iap;
@property (nonatomic, copy) IAPReceiptDataBlock receiptDataHandlerBlock;

@end

@implementation IAPManager

@synthesize receiptDataHandlerBlock;

- (void)getSKProducts:(void (^)(NSError *))completion {
    
    [self.iap requestProductsWithCompletion:^(SKProductsRequest *request, SKProductsResponse *responce){
        if (responce == nil) {
            RUN_BLOCK_SAFE(completion, [NSError errorWithDomain:@"IAP" code:-1 userInfo:@{NSLocalizedDescriptionKey:@"Can't get price info"}]);
            return ;
        }
        
        if (responce.products.count) {
            for (SKProduct *skProduct in responce.products) {
                for (IAPProduct *iapProduct in _products) {
                    if ([iapProduct.productId isEqualToString:skProduct.productIdentifier]) {
                        iapProduct.skProduct = skProduct;
                        break;
                    }
                }
            }
        }
        RUN_BLOCK_SAFE(completion, nil);
    }];
}

- (void)getSubscribedPeriods:(void(^)(NSArray *periods, NSError *error))completion{
    
    if ([self.delegate respondsToSelector:@selector(getNonRenewableSubscriptionPurchasedPeriods:)]) {
        
        [self.delegate getNonRenewableSubscriptionPurchasedPeriods:^(NSArray *periods, NSError *error){
            
            if (error)
                RUN_BLOCK_SAFE(completion, nil, error);
            else
                [self getSubscribedPeriodsWithNonRenewableSubscriptionPurchasedPeriods:periods
                                                                            completion:completion];
        }];
    }
    else
        [self getSubscribedPeriodsWithNonRenewableSubscriptionPurchasedPeriods:[NSArray new]
                                                                    completion:completion];
    
}

- (void)getSubscribedPeriodsWithNonRenewableSubscriptionPurchasedPeriods:(NSArray*) nonRenewablePeriods
                                                              completion:(void(^)(NSArray *periods, NSError *error))completion{

    [self getReceiptData:^(NSData *_receiptData, NSError *error){
        
        if (!error) {
            
            NSString *sharedSecret = [Config iTunesSharedSecret];
            
            [self.iap checkReceipt:_receiptData AndSharedSecret:sharedSecret
                      onCompletion:^(NSString *responceJSONString, NSError *error){
                          
                          if (!error) {
                              
                              NSData *data = [responceJSONString dataUsingEncoding:NSUTF8StringEncoding];
                              NSDictionary *jsonResponse = [NSJSONSerialization JSONObjectWithData:data
                                                                                           options:kNilOptions
                                                                                             error:&error];
                              
                              NSMutableArray *receipts = [jsonResponse[@"latest_receipt_info"] mutableCopy];
                              
                              NSMutableArray *result = [NSMutableArray new];
                              
                              if (nonRenewablePeriods.count)
                                  [result addObjectsFromArray:nonRenewablePeriods];
                              
                              [receipts enumerateObjectsUsingBlock:^(NSDictionary *receiptDictionary, NSUInteger idx, BOOL *stop) {
                                  
                                  IAPPeriod *iapPeriod = [IAPPeriod new];
                                  
                                  NSNumber *purchaseDateMs = receiptDictionary[@"purchase_date_ms"];
                                  //NSString *productId = receiptDictionary[@"product_id"];
                                  
                                  if(purchaseDateMs && ![purchaseDateMs isKindOfClass: [NSNull class]]){
                                      
                                      NSDate *startDate = [NSDate dateWithTimeIntervalSince1970:purchaseDateMs.doubleValue/1000];
                                      iapPeriod.startDate = startDate;
                                  }
                                  
                                  NSNumber *expiresDateMs = receiptDictionary[@"expires_date_ms"];
                                  
                                  if(expiresDateMs && ![expiresDateMs isKindOfClass: [NSNull class]]){
                                      
                                      NSDate *endDate = [NSDate dateWithTimeIntervalSince1970:expiresDateMs.doubleValue/1000];
                                      iapPeriod.endDate = endDate;
                                  }
                                  
                                  [result addObject:iapPeriod];
                              }];
                              
                              RUN_BLOCK_SAFE(completion,result, error);
                          }
                          else
                              RUN_BLOCK_SAFE(completion,nil, error);
                      }];
        }
        else
            RUN_BLOCK_SAFE(completion, NO, error);
    }];
}

-(NSArray*) paidProducts{
    
    NSMutableArray *result = [NSMutableArray new];
    
    for (IAPProduct *product in _products) {
        
        if ([product type]==IAPTypeSubscriptionAutoRenewable||[product type]==IAPTypeSubscriptionNonRenewable)
        [result addObject:product];
    }
    
    return result;
}

- (void)isSubscribedNow:(void (^)(BOOL, NSDate *, NSError *))completion {
    
    NSNumber *expiryDateNumber = [self greatestExpiryDateNumber];
    
    if ([expiryDateNumber doubleValue] != 0) {
        
        NSTimeInterval currentTimeInterval = [[NSDate date] timeIntervalSince1970];
        NSTimeInterval expiryDateInterval = [expiryDateNumber doubleValue];
        
        if (currentTimeInterval < expiryDateInterval) {
            
            RUN_BLOCK_SAFE(completion, YES, [NSDate dateWithTimeIntervalSince1970:expiryDateInterval], nil);
            return;
        }
    }
    
    [self getSubscribedPeriods:^(NSArray *periods, NSError *error){
        
        if (!error) {
            
            __block NSDate *greatestDate = nil;
            
            NSArray *paidProducts = [self paidProducts];
            
            if (paidProducts.count && periods.count == 0) {
                
                RUN_BLOCK_SAFE(completion, NO, nil, nil);
                return ;
            }
            
            [periods enumerateObjectsUsingBlock:^(IAPPeriod *period, NSUInteger idx, BOOL *stop) {
                
                if (greatestDate == nil )
                    greatestDate = period.endDate;
               
                else{
                    
                    if (period.endDate) {
                        
                        NSTimeInterval greatesTtimeInterval = [greatestDate timeIntervalSince1970];
                        NSTimeInterval endDateTimeInterval = [period.endDate timeIntervalSince1970];
                        
                        if (endDateTimeInterval>greatesTtimeInterval) {
                            greatestDate = period.endDate;
                        }
                    }
                }
            }];
            
            if (greatestDate) {
                
                NSTimeInterval greatestTimeInterval = [greatestDate timeIntervalSince1970];
                NSNumber *storedDateNumber = [NSNumber numberWithDouble:greatestTimeInterval];
                
                NSData *numberAsData = [NSKeyedArchiver archivedDataWithRootObject:storedDateNumber];
                [numberAsData writeToFile:[self expiryDateRecordFilePath] atomically:YES];
                NSTimeInterval currentTimeInterval = [[NSDate date] timeIntervalSince1970];
                
                RUN_BLOCK_SAFE(completion, currentTimeInterval<greatestTimeInterval, greatestDate, nil);
            }
            else
                paidProducts.count ? RUN_BLOCK_SAFE(completion, NO, nil, nil) : RUN_BLOCK_SAFE(completion, YES, greatestDate, nil);
        }
        else {
            RUN_BLOCK_SAFE(completion, NO, nil, error);
        }
    }];
}

- (BOOL)isPurchased:(IAPProduct *)product {
#if TARGET_IPHONE_SIMULATOR
    return YES;
#endif
    
    return [self.iap isPurchasedProductsIdentifier:product.productId];
}

- (void)isPurchased:(IAPProduct *)product completion:(void (^)(BOOL, NSError *))completion {
    
    if (product.type==IAPTypeSubscriptionNonRenewable ||
        product.type==IAPTypeSubscriptionAutoRenewable) {
        
        if ([self.delegate respondsToSelector:@selector(isPurchasedNonRenewableSubscription:completion:)]) {
            
            [self.delegate isPurchasedNonRenewableSubscription:product completion:^(BOOL purchased, NSError *error){
                
                RUN_BLOCK_SAFE(completion,purchased,error);
            }];
        }
        else
            RUN_BLOCK_SAFE(completion,NO,nil);
        
        return;
    }
    
    BOOL result = [self.iap isPurchasedProductsIdentifier:[product productId]];
    RUN_BLOCK_SAFE(completion, result, nil);
}

-(NSString*) expiryDateRecordFilePath {
    
    NSString *documentDirectory = NSSearchPathForDirectoriesInDomains(NSCachesDirectory,
                                                                      NSUserDomainMask, YES).firstObject;
    return [documentDirectory stringByAppendingPathComponent:@"expdr"];
}

-(NSNumber*) greatestExpiryDateNumber{
    
    return [NSKeyedUnarchiver unarchiveObjectWithFile:[self expiryDateRecordFilePath]];
}

- (void)getReceiptData:(void(^)(NSData *receiptData, NSError *error))completion {
#if TARGET_IPHONE_SIMULATOR
    NSError *error = [NSError errorWithDomain:@"IAPManager" code:-1 userInfo:@{NSLocalizedDescriptionKey:@"Purchases not supported in Simulator"}];
    RUN_BLOCK_SAFE(completion, nil, error);
    return;
#endif
    
    NSURL *receiptUrl = [[NSBundle mainBundle] appStoreReceiptURL];
    if ([[NSFileManager defaultManager] fileExistsAtPath:[receiptUrl path]]) {
        NSData *receiptData = [NSData dataWithContentsOfURL:receiptUrl];
        RUN_BLOCK_SAFE(completion, receiptData, nil);
    }
    else {
        self.receiptDataHandlerBlock = completion;
        SKReceiptRefreshRequest *refreshReceiptRequest = [[SKReceiptRefreshRequest alloc] initWithReceiptProperties:@{}];
        refreshReceiptRequest.delegate = self;
        [refreshReceiptRequest start];
    }
}

- (void)purchaseProduct:(IAPProduct *)product completion:(void (^)(NSError *error))completion {
    
    if (product.type == IAPTypeSubscriptionNonRenewable
        || product.type == IAPTypeSubscriptionAutoRenewable) {
        
        [self isSubscribedNow:^(BOOL isSubscribed, NSDate *endDate, NSError *error) {
            if (!error) {
                if (isSubscribed) {
                    NSDictionary *errorDetail = @{NSLocalizedDescriptionKey:@"User has been already subscribed"};
                    NSError *error = [NSError errorWithDomain:@"IAPManager" code:100 userInfo:errorDetail];
                    RUN_BLOCK_SAFE(completion, error);
                    
                    return ;
                }
                
                [self checkSKProductAndPurchaseProduct:product completion:completion];
            }
            else
                RUN_BLOCK_SAFE(completion, error);
        }];
    }
    else {
        [self checkSKProductAndPurchaseProduct:product completion:completion];
    }
}

-(void) checkSKProductAndPurchaseProduct:(IAPProduct *)product completion:(void (^)(NSError *))completion{
    
    [product getSKProduct:^(NSError *error){
        
        if (error) {
            RUN_BLOCK_SAFE(completion, error);
            return ;
        }
        
        [self.iap buyProduct:[product skProduct] onCompletion:^(SKPaymentTransaction *transaction){
            
            if (transaction.error)
                RUN_BLOCK_SAFE(completion,transaction.error);
            else
                RUN_BLOCK_SAFE(completion, nil);
        }];
    }];
}

- (void)restorePurchases:(void (^)(NSError *))completion {
    
    [self.iap restoreProductsWithCompletion:^(SKPaymentQueue *payment, NSError *error) {
        RUN_BLOCK_SAFE(completion, error);
    }];
}

#pragma mark - Properties
- (void)setProducts:(NSArray *)products {
    NSMutableSet *ids = [NSMutableSet set];
    [products enumerateObjectsUsingBlock:^(IAPProduct *obj, NSUInteger idx, BOOL *stop) {
        [ids addObject:obj.productId];
    }];
    
    _products  = products;
    
    self.iap = [[IAPHelper alloc] initWithProductIdentifiers:ids];
    
    if ([[NSUserDefaults standardUserDefaults] objectForKey:@"lasttDate"] == nil) {
        [[NSUserDefaults standardUserDefaults] setObject:@"today" forKey:@"lasttDate"];
        [[NSUserDefaults standardUserDefaults] synchronize];
        [self.iap clearSavedPurchasedProducts];
    }
    
#warning RS: set Production mode here
    self.iap.production = NO;
    return;
    [self.iap requestProductsWithCompletion:^(SKProductsRequest *request, SKProductsResponse *response) {
        for (SKProduct *skProduct in response.products) {
            for (IAPProduct *product in self.products) {
                if ([product.productId isEqualToString:skProduct.productIdentifier]) {
                    product.skProduct = skProduct;
                }
            }
        }
        
    }];
}

#pragma mark -
#pragma mark - SKRequestDelegate

-(void) requestDidFinish:(SKRequest *)request{
    
    if([request isKindOfClass:[SKReceiptRefreshRequest class]])
    {
        NSURL *receiptUrl = [[NSBundle mainBundle] appStoreReceiptURL];
        
        if ([[NSFileManager defaultManager] fileExistsAtPath:[receiptUrl path]]) {
            NSData *receiptData = [NSData dataWithContentsOfURL:receiptUrl];
            
            IAPReceiptDataBlock tempBlock = self.receiptDataHandlerBlock;
            self.receiptDataHandlerBlock = nil;
            RUN_BLOCK_SAFE(tempBlock, receiptData, nil);
        }
    }
}

-(void) request:(SKRequest *)request didFailWithError:(NSError *)error{
    IAPReceiptDataBlock tempBlock = self.receiptDataHandlerBlock;
    self.receiptDataHandlerBlock = nil;
    RUN_BLOCK_SAFE(tempBlock, nil, error);
}

@end

