//
//  MagIssue.m
//  Baker
//
//  Created by Tope on 19/06/2012.
//  Copyright (c) 2012 AppDesignVault. All rights reserved.
//


#import "Issue.h"
#import "Config.h"
#import <NewsstandKit/NewsstandKit.h>
#import "SSZipArchive.h"


@interface Issue () <NSURLConnectionDownloadDelegate, NSURLSessionDownloadDelegate>

@property (nonatomic, strong) NSString *uniqueId;
@property (nonatomic, strong) NSString *title;
@property (nonatomic, strong) NSString *descriptionText;
@property (nonatomic, strong) NSDate *publicationDate;
@property (nonatomic, strong) NSURL *coverImageUrl;
@property (nonatomic, strong) NSURL *contentUrl;
@property (nonatomic, assign) IssueType type;
@property (nonatomic, strong) NSString *inAppPurchaseId;
@property (nonatomic, strong) NSArray *previewImageURLs;

@property (nonatomic, strong) IAPProduct *product;
@property (nonatomic, strong) NKIssue *nkIssue;

@property (nonatomic, strong) NSURLSession *session;

@property (nonatomic, assign) BOOL paidContent;
@property (nonatomic, assign) BOOL paidIfSubscribed;

@property (nonatomic, copy) void(^downloadCompletion)(NSError *error);

@end


@implementation Issue

+ (NSArray *)issuesWithArray:(NSArray *)array {
    NSMutableArray *result = [NSMutableArray array];
    for (NSDictionary *dict in array) {
        [result addObject:[[Issue alloc] initWithDictionary:dict]];
    }
    
    return result;
}

- (instancetype)initWithDictionary:(NSDictionary *)dict {
    self = [super init];
    
    self.uniqueId = dict[@"uniqueId"];
    self.title = dict[@"title"];
    self.descriptionText = dict[@"description"];
    
    NSDateFormatter *df = [[NSDateFormatter alloc] init];
    df.dateFormat = @"dd-MM-YYYY";
    df.locale = [NSLocale localeWithLocaleIdentifier:@"en"];
    self.publicationDate = [df dateFromString:dict[@"date"]];
    
    self.coverImageUrl = [NSURL URLWithString:dict[@"coverImageUrl"]];
    
    if ([UIDevice currentDevice].userInterfaceIdiom == UIUserInterfaceIdiomPad) {
        self.contentUrl = [NSURL URLWithString:dict[@"contentUrlIpad"]];
    }
    else {
        self.contentUrl = [NSURL URLWithString:dict[@"contentUrl"]];
    }
    
    NSMutableArray *temp = [NSMutableArray array];
    for (NSString *s in dict[@"preview"]) {
        [temp addObject:[NSURL URLWithString:s]];
    }
    self.previewImageURLs = [temp copy];
    
    NSString *typeString = dict[@"type"];
    if ([UIDevice currentDevice].userInterfaceIdiom == UIUserInterfaceIdiomPad) {
        typeString = dict[@"typeIpad"];
    }
    IssueType type = IssueTypeUnknown;
    if ([[typeString lowercaseString] isEqualToString:@"pdf"]) {
        type = IssueTypePDF;
    }
    else if ([[typeString lowercaseString] isEqualToString:@"hpub"]) {
        type = IssueTypeHPUB;
    }
    else if ([[typeString lowercaseString] isEqualToString:@"html"]) {
        type = IssueTypeHTML;
    }
    self.type = type;
    
    self.inAppPurchaseId = dict[@"in-app-purchase-id"];
    self.paidContent = (self.inAppPurchaseId != nil);
    
    self.paidIfSubscribed = [dict[@"paidIfSubscribed"] boolValue];
    
    self.product = [[IAPProduct alloc] initWithId:self.inAppPurchaseId type:IAPTypeNonConsumable];
    
    if ([Config isNewsstandApp]) {
        // set nkIssue
        NKIssue *nkIssue = [[NKLibrary sharedLibrary] issueWithName:self.title];
        if (nkIssue == nil) {
            nkIssue = [[NKLibrary sharedLibrary] addIssueWithName:self.title date:self.publicationDate];
        }
        self.nkIssue = nkIssue;
    }
    
    self.status = IssueStatusNotPurchased;
    
    if ([[Publisher sharedInstance] isPurchasedIssue:self]) {
        self.status = IssueStatusNotDownloaded;
        
        if ([Config isNewsstandApp]) {
            for (NKAssetDownload *asset in [[NKLibrary sharedLibrary] downloadingAssets]) {
                if ([asset.issue isEqual:self.nkIssue]) {
                    [asset downloadWithDelegate:self];
                    self.status = IssueStatusDownloading;
                }
            }
            
            if (self.nkIssue.status == NKIssueContentStatusAvailable) {
                self.downloadProgress = 1.0;
                self.status = IssueStatusReady;
            }
        }
        else {
            if (self.downloadedPath != nil) {
                self.downloadProgress = 1.0;
                self.status = IssueStatusReady;
            }
        }
    }
    
    return self;
}

#pragma mark -
- (NSString *)downloadedPath {
    NSString *result = self.downloadToPath;
    if (![[NSFileManager defaultManager] fileExistsAtPath:result]) {
        result = nil;
    }
    
    return result;
}

- (NSString *)downloadToPath {
    NSString *cachePath = nil;
    
    if ([Config isNewsstandApp]) {
        cachePath = self.nkIssue.contentURL.path;
    }
    else {
        NSArray *paths = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
        cachePath = [paths objectAtIndex:0];
        cachePath = [cachePath stringByAppendingPathComponent:@"issues"];
        
        if (![[NSFileManager defaultManager] fileExistsAtPath:cachePath]) {
            [[NSFileManager defaultManager] createDirectoryAtPath:cachePath withIntermediateDirectories:YES attributes:nil error:nil];
        }
    }
    
    NSString *result = [cachePath stringByAppendingPathComponent:self.uniqueId];
    return result;
}

- (void)downloadIssue:(void(^)(NSError *error))completion {
    if (self.contentUrl == nil) {
        NSError *error = [NSError errorWithDomain:@"AppLogic" code:-1 userInfo:@{NSLocalizedDescriptionKey:[NSString stringWithFormat:@"Content URL is absent for %@", self.title]}];
        RUN_BLOCK_SAFE(completion, error);
        return;
    }
    
    self.downloadCompletion = completion;
    
    if (self.status == IssueStatusDownloading) {
        NSLog(@"WARNING: Call downloadIssue while in progress");
        return;
    }
    
    if ([Config isNewsstandApp]) {
        NSURLRequest *request = [NSURLRequest requestWithURL:self.contentUrl];
        NKAssetDownload *assetDownload = [self.nkIssue addAssetWithRequest:request];
        [assetDownload downloadWithDelegate:self];
        self.status = IssueStatusDownloading;
    }
    else {
        NSURLRequest *request = [NSURLRequest requestWithURL:self.contentUrl];
        self.session = [NSURLSession sessionWithConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration] delegate:self delegateQueue:nil];
        NSURLSessionDownloadTask *task = [self.session downloadTaskWithRequest:request];
        [task resume];
        self.status = IssueStatusDownloading;
    }
}

- (void)deleteDownload {
    if ([Config isNewsstandApp]) {
        [[NKLibrary sharedLibrary] removeIssue:self.nkIssue];
        self.nkIssue = [[NKLibrary sharedLibrary] addIssueWithName:self.title date:self.publicationDate];
    }
    else {
        [self.session invalidateAndCancel];
        self.session = nil;
        [[NSFileManager defaultManager] removeItemAtPath:[self downloadToPath] error:nil];
    }
    
    if (self.status == IssueStatusDownloading || self.status == IssueStatusReady) {
        self.status = IssueStatusNotDownloaded;
    }
}

#pragma mark - NSURLConnectionDownloadDelegate
- (void)updateProgressWithTotalBytesWritten:(long long)totalBytesWritten expectedTotalBytes:(long long)expectedTotalBytes {
    self.downloadProgress = 1.0*totalBytesWritten/expectedTotalBytes;
}

- (void)connection:(NSURLConnection *)connection didWriteData:(long long)bytesWritten totalBytesWritten:(long long)totalBytesWritten expectedTotalBytes:(long long)expectedTotalBytes {
    [self updateProgressWithTotalBytesWritten:totalBytesWritten expectedTotalBytes:expectedTotalBytes];
}

- (void)connectionDidResumeDownloading:(NSURLConnection *)connection totalBytesWritten:(long long)totalBytesWritten expectedTotalBytes:(long long)expectedTotalBytes {
    NSLog(@"Resume downloading %f", 1.f*totalBytesWritten/expectedTotalBytes);
    [self updateProgressWithTotalBytesWritten:totalBytesWritten expectedTotalBytes:expectedTotalBytes];
}

- (void)connectionDidFinishDownloading:(NSURLConnection *)connection destinationURL:(NSURL *)destinationURL {
    [self updateProgressWithTotalBytesWritten:1.0 expectedTotalBytes:1.0];
    // copy file to destination URL
    NSString *contentPath = [self downloadToPath];
    
    NSError *moveError = nil;
    if (self.type == IssueTypePDF) {
        [[NSFileManager defaultManager] moveItemAtPath:[destinationURL path] toPath:contentPath error:&moveError];
        if (moveError != nil) {
            NSLog(@"Error copying file from %@ to %@", destinationURL, contentPath);
            self.status = IssueStatusNotDownloaded;
            RUN_BLOCK_SAFE(self.downloadCompletion, moveError);
        }
        else {
            self.status = IssueStatusReady;
            RUN_BLOCK_SAFE(self.downloadCompletion, nil);
        }
    }
    else if (self.type == IssueTypeHPUB || self.type == IssueTypeHTML) {
        [SSZipArchive unzipFileAtPath:destinationURL.path toDestination:contentPath overwrite:YES password:nil error:&moveError];
    }
    
    
    if (moveError != nil) {
        NSLog(@"Error copying file from %@ to %@", destinationURL, contentPath);
        self.status = IssueStatusNotDownloaded;
        RUN_BLOCK_SAFE(self.downloadCompletion, moveError);
    }
    else {
        self.status = IssueStatusReady;
        RUN_BLOCK_SAFE(self.downloadCompletion, nil);
    }
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
    [self updateProgressWithTotalBytesWritten:0.0 expectedTotalBytes:1.0];
    self.status = IssueStatusNotDownloaded;
    RUN_BLOCK_SAFE(self.downloadCompletion, error);
}

#pragma mark - NSURLSessionDownloadDelegate
- (void)URLSession:(NSURLSession *)session downloadTask:(NSURLSessionDownloadTask *)downloadTask didFinishDownloadingToURL:(NSURL *)location {
    [self updateProgressWithTotalBytesWritten:1.0 expectedTotalBytes:1.0];
    // copy file to destination URL
    NSString *contentPath = [self downloadToPath];
    
    NSError *moveError = nil;
    if (self.type == IssueTypePDF) {
        [[NSFileManager defaultManager] moveItemAtPath:location.path toPath:contentPath error:&moveError];
        if (moveError != nil) {
            NSLog(@"Error copying file from %@ to %@", location, contentPath);
            self.status = IssueStatusNotDownloaded;
            RUN_BLOCK_SAFE(self.downloadCompletion, moveError);
        }
        else {
            self.status = IssueStatusReady;
            RUN_BLOCK_SAFE(self.downloadCompletion, nil);
        }
    }
    else if (self.type == IssueTypeHPUB || self.type == IssueTypeHTML) {
        [SSZipArchive unzipFileAtPath:location.path toDestination:contentPath overwrite:YES password:nil error:&moveError];
    }
    
    
    if (moveError != nil) {
        NSLog(@"Error copying file from %@ to %@", location, contentPath);
        self.status = IssueStatusNotDownloaded;
        RUN_BLOCK_SAFE(self.downloadCompletion, moveError);
    }
    else {
        self.status = IssueStatusReady;
        RUN_BLOCK_SAFE(self.downloadCompletion, nil);
    }
    self.session = nil;
}

- (void)URLSession:(NSURLSession *)session downloadTask:(NSURLSessionDownloadTask *)downloadTask didWriteData:(int64_t)bytesWritten totalBytesWritten:(int64_t)totalBytesWritten totalBytesExpectedToWrite:(int64_t)totalBytesExpectedToWrite {
    [self updateProgressWithTotalBytesWritten:totalBytesWritten expectedTotalBytes:totalBytesExpectedToWrite];
}

- (void)URLSession:(NSURLSession *)session downloadTask:(NSURLSessionDownloadTask *)downloadTask didResumeAtOffset:(int64_t)fileOffset expectedTotalBytes:(int64_t)expectedTotalBytes {
    NSLog(@"Resume downloading %f", 1.f*fileOffset/expectedTotalBytes);
    [self updateProgressWithTotalBytesWritten:fileOffset expectedTotalBytes:expectedTotalBytes];
}

- (void)URLSession:(NSURLSession *)session task:(NSURLSessionTask *)task didCompleteWithError:(NSError *)error {
    if (error != nil) {
        NSLog(@"Error download file from %@, %@", self.contentUrl, error);
        self.status = IssueStatusNotDownloaded;
        RUN_BLOCK_SAFE(self.downloadCompletion, error);
    }
    self.session = nil;
}

@end

